/****************************************************************************************

		IMCUndo.h

		Copyright: (c) 2001 Eovia Corp. All rights reserved.


		Author:	Nicolas
		Date:	12/28/98

		Revision History

		when		who			what & why
		==========	==========	======================================================

*****************************************************************************************/
#ifndef __IMCUndo__
#define __IMCUndo__

#if CP_PRAGMA_ONCE
#pragma once
#endif

#include "COMTypes.h"

//****** Globally Unique Ids *************************************************

MCDEFINE_GUID(IID_IMCUndo, 0xb311f590, 0x9e8d, 0x11d2, 0x8e, 0xbe, 0x0, 0x10, 0x4b, 0x2d, 0x8e, 0x67);


/*****************************************************************************\
*                                                                             *
*  IMCUndo                                                                    *
*                                                                             *
\*****************************************************************************/

/// \addtogroup MCImage
/// \{

/// IMCUndo is used to under modifications in images. It is implemented by
/// raster layers (IShRasterLayer) and channels (see IShChannel).
///
struct IMCUndo : IMCUnknown
{
	// ****************************************************************************
	//								StartUndoableSession
	// 
	/// Increments the version number, and returns it 
	/// it's the version being edited
	/// it's safe to nest the calls 
	/// (non top levels will just return the current version)
	/// 
	virtual uint32	 MCCOMAPI StartUndoableSession() = 0; 
	
	// ****************************************************************************
	//								FinishUndoableSession
	// 
	/// 
	virtual void	 MCCOMAPI FinishUndoableSession() = 0;

	// ****************************************************************************
	//								RemoveVersionSmallerThan
	// 
	/// removes versions that are smaller (strictly) than minVersion 
	/// (for a max undo stack)
	///
	/// \param minVersion  
	/// 
	virtual void	 MCCOMAPI RemoveVersionSmallerThan(uint32 minVersion) = 0;

	// ****************************************************************************
	//								RemoveVersionBiggerThan
	// 
	/// Removes versions that are bigger (strictly) than maxVersion
	/// 
	/// \param maxVersion  
	/// 
	virtual void	 MCCOMAPI RemoveVersionBiggerThan(uint32 maxVersion) = 0; 

	// ****************************************************************************
	//								GetCurrentVersion
	// 
	/// 
	virtual uint32	 MCCOMAPI GetCurrentVersion() = 0;
	
	// ****************************************************************************
	//								GotoVersion
	// 
	/// 
	virtual MCCOMErr MCCOMAPI GotoVersion(uint32 newVersion) = 0;
};

/// \}

#endif